####################################################################
#
#  Implements a context in which the student's answer is treated as a
#  literal string, and not parsed further.  The real answer checking
#  should be performed in a custom checker passed to the answer
#  string's cmp() method.  E.g.,
#
#        loadMacros("contextArbitraryString.pl");
#        Context("ArbitraryString");
#
#        ANS(Compute("The string I want")->cmp(checker => sub {
#          my ($correct,$student,$ans) = @_;
#          $correct = $correct->value; # get perl string from String object
#          $student = $student->value; # ditto
#          ##
#          ## do your checking here, and return true if correct
#          ## or false if incorrect.  For example
#          ##   return $correct eq $student;
#          ##
#          return $score;
#        }));
#
#  If you want students to be able to include characters like $ or @ in their answers,
#  then you should also add
#
#	loadMacros("problemPreserveAnswers.pl");
#
#  at the beginning of your problem.
#

sub _contextArbitraryString_init {
  my $context = $main::context{ArbitraryString} = Parser::Context->getCopy("Numeric");
  $context->{name} = "ArbitraryString";
  $context->parens->clear();
  $context->variables->clear();
  $context->constants->clear();
  $context->operators->clear();
  $context->functions->clear();
  $context->strings->clear();
  $context->{pattern}{number} = "^\$";
  $context->variables->{patterns} = {};
  $context->strings->{patterns}{"(.|\n)*"} = [-20,'str'];
  $context->{value}{"String()"} = "context::ArbitraryString";
  $context->{value}{"String"} = "context::ArbitraryString::Value::String";
  $context->{parser}{String} = "context::ArbitraryString::Parser::String";
  $context->update;
}

package context::ArbitraryString;
sub new {shift; main::Compute(@_)}

package context::ArbitraryString::Parser::String;
our @ISA = ('Parser::String');

sub new {
  my $self = shift;
  my ($equation,$value,$ref) = @_;
  $value = $equation->{string};
  $self->SUPER::new($equation,$value,$ref);
}

package context::ArbitraryString::Value::String;
our @ISA = ("Value::String");

#
#  Mark a string to be display verbatim
#
sub verb {return "\\verb".chr(126).(shift).chr(126)}

#
#  Mark a multi-line string to be displayed verbatim in TeX
#
sub quoteTeX {
  my $s = shift;
  return verb($s) unless $s =~ m/\n/;
  my @tex = split(/\n/,$s);
  foreach (@tex) {$_ = verb($_) if $_ =~ m/\S/}
  "\\begin{array}{l}".join("\\\\ ",@tex)."\\end{array}";
}

#
#  Render the value verbatim
#
sub TeX {
  my $self = shift;
  quoteTeX($self->value);
}

#
#  Include the correct_ans_latex_string as the properly-displayed
#  verbatim correct answer (or string value)
#
sub cmp {
  my $self = shift;
  $self->SUPER::cmp(
    correct_ans_latex_string => quoteTeX($self->{correct_ans}||$self->string),
    @_
  );
}

#
#  Compare normalized versions of the two values
#  Here, we normalize line breaks, but could also
#  do white-space trimming or collapsing as well.
#
sub compare {
  my ($self,$l,$r) = Value::checkOpOrder(@_);
  $l =~ s/\r\n?/\n/g;
  $r =~ s/\r\n?/\n/g;
  return $l cmp $r;
}

1;
